"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Command = void 0;
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = require("commander");
const figlet_1 = __importDefault(require("figlet"));
const constants_1 = require("../constants");
const factories_1 = require("../factories");
const services_1 = require("../services");
const storage_1 = require("../services/storage");
const utils_1 = require("../utils");
const urlCommand_1 = require("./urlCommand");
/**
 * Command extension class
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
class Command extends commander_1.Command {
    constructor() {
        super();
        this.actions = [];
        this.commandOptions = {};
    }
    initialize(initializeAction) {
        initializeAction();
        return this;
    }
    optionInteractive(optionConfig) {
        const { shortName, longName, description, defaultValue, configKey, prompt, choices } = optionConfig;
        const variableName = this.getVariableName(longName);
        const decoratedDescription = description +
            (configKey ? `\nCan be provided via environment variable ${utils_1.Config.getEnvironmentVariableName(configKey)}` : "") +
            "\nCan be provided interactively by user if not available";
        this.option(`${shortName}, ${longName} <${variableName}>`, decoratedDescription);
        // Add action for getting option value
        return this.addAction((serviceCollection) => __awaiter(this, void 0, void 0, function* () {
            const currentValue = this.opts()[variableName];
            // If defined, means they've passed in an argument
            if (currentValue !== undefined) {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                this.commandOptions[variableName] = currentValue;
                return;
            }
            if (configKey) {
                // Try to get value from configuration - return if exists
                const configuredVal = utils_1.Config.getValueWithDefault(configKey);
                if (configuredVal) {
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any
                    this.commandOptions[variableName] = configuredVal;
                    return;
                }
            }
            const questionPrompt = prompt || `Enter a value for ${longName}:`;
            // Ask user for value - use multiple choice if applicable
            const answeredVal = choices
                ? yield utils_1.UserUtils.multiChoiceQuestion(questionPrompt, choices)
                : yield utils_1.UserUtils.askQuestion(questionPrompt, defaultValue);
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            this.commandOptions[variableName] = answeredVal;
            // Ask user if they'd like to save to .env
            if (configKey &&
                (yield utils_1.UserUtils.confirmAction(`Would you like to save to your local ${constants_1.FileConstants.envFileName} file?`))) {
                const { contentService } = serviceCollection;
                const currentEnvFile = yield contentService.getIfExists(constants_1.FileConstants.envFileName);
                const updatedEnvFile = utils_1.Config.getUpdatedEnvFileContents(configKey, answeredVal, currentEnvFile);
                yield contentService.set(constants_1.FileConstants.envFileName, updatedEnvFile);
            }
        }));
    }
    /**
     * Gets a variable name from a long option name
     *
     * @param {string} longName Long variable name `--my-variable-name`
     * @returns {string} camelCasedVariableName (e.g. myVariableName)
     */
    getVariableName(longName) {
        const longNameSplit = longName.replace(/^--/, "").split("-");
        if (longNameSplit.length === 1) {
            return longNameSplit[0];
        }
        const capitalizeFirstLetter = (word) => {
            const firstChar = word.charAt(0);
            return word.replace(firstChar, firstChar.toUpperCase());
        };
        return longNameSplit.map((value, index) => (index > 0 ? capitalizeFirstLetter(value) : value)).join("");
    }
    addAction(actionHandler) {
        this.actions.push(actionHandler);
        this.action(() => __awaiter(this, void 0, void 0, function* () {
            this.commandOptions = Object.assign(Object.assign({}, this.opts()), this.commandOptions);
            const storage = new storage_1.FileStorageService(process.cwd());
            const configObjectService = new services_1.ObjectService(storage);
            const fileConfig = yield configObjectService.getIfExists(constants_1.FileConstants.configFileName);
            const config = services_1.ConfigService.overrideConfigWithOptions(fileConfig, this.commandOptions);
            const serviceCollection = yield factories_1.ServiceCollectionFactory.create(config);
            for (const action of this.actions) {
                yield action(serviceCollection, this.commandOptions, config);
            }
        }));
        return this;
    }
    printHelp() {
        this.addAction(() => {
            utils_1.Logger.log(this.helpInformation());
        });
        return this;
    }
    asciiArt(message) {
        this.addAction(() => {
            utils_1.Logger.log(chalk_1.default.cyanBright(figlet_1.default.textSync(message)));
        });
        return this;
    }
    addLinkCommands(links) {
        links.forEach((link) => this.addCommand(urlCommand_1.urlCommand(link)));
        return this;
    }
    addPhoneTree() {
        this.addAction(() => __awaiter(this, void 0, void 0, function* () {
            const subCommandNames = this.commands.map((command) => `${command.name()} - ${command.description()}`);
            const subCommandNameChosen = (yield utils_1.UserUtils.multiChoiceQuestion(this.name(), subCommandNames)).split(" - ")[0];
            const subCommandChosen = this.commands.find((command) => command.name() === subCommandNameChosen);
            if (subCommandChosen) {
                yield subCommandChosen.parseAsync(this.args);
            }
        }));
        return this;
    }
    toMarkdownDoc(parentCommandPath) {
        const commandPath = `${parentCommandPath ? parentCommandPath + " " : ""}${this.name()}`;
        let doc = `## \`${commandPath}\`\n\n`;
        doc += "```\n" + this.helpInformation() + "```";
        for (const command of this.commands) {
            if (command instanceof Command) {
                doc += `\n${command.toMarkdownDoc(commandPath)}`;
            }
        }
        return doc;
    }
    toTableOfContents(linkPrefix, parentIndentation, parentCommandPath) {
        const indentation = parentIndentation !== undefined ? parentIndentation + "  " : "";
        const commandPath = `${parentCommandPath ? parentCommandPath + " " : ""}${this.name()}`;
        const line = "- [`" +
            commandPath +
            "`]" +
            `(${linkPrefix ? linkPrefix : ""}#${commandPath.replace(/ /g, "-")}) - ${this.description()}`;
        let toc = indentation + line;
        for (const command of this.commands) {
            if (command instanceof Command) {
                toc += `\n${command.toTableOfContents(linkPrefix, indentation, commandPath)}`;
            }
        }
        return toc;
    }
}
exports.Command = Command;
