"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzureDevOpsUtils = void 0;
const constants_1 = require("../constants");
const models_1 = require("../models");
const azureDevOpsFieldName_1 = require("../services/agile/providers/azureDevOps/azureDevOpsFieldName");
const azureDevOpsWorkItemType_1 = require("../services/agile/providers/azureDevOps/azureDevOpsWorkItemType");
const jsonPatchDocument_1 = require("../services/agile/providers/azureDevOps/jsonPatchDocument");
const configUtils_1 = require("./configUtils");
class AzureDevOpsUtils {
    static createPatchDocument(backlogItem, parentUrl) {
        const pairs = this.createFieldValuePairs(backlogItem);
        // Only include pairs whose value is defined
        const document = pairs
            .filter((pair) => pair.value)
            .map((pair) => {
            return {
                op: jsonPatchDocument_1.AzDoOp.Add,
                path: `/fields/${pair.field}`,
                value: pair.value,
            };
        });
        if (parentUrl) {
            document.push({
                op: jsonPatchDocument_1.AzDoOp.Add,
                path: "/relations/-",
                value: {
                    rel: "System.LinkTypes.Hierarchy-Reverse",
                    url: parentUrl,
                    attributes: {
                        isLocked: false,
                        name: "Parent",
                    },
                },
            });
        }
        return document;
    }
    static getWorkItemType(backlogItemType) {
        switch (backlogItemType) {
            case models_1.BacklogItemType.Task:
                return azureDevOpsWorkItemType_1.AzureDevOpsWorkItemType.Task;
            case models_1.BacklogItemType.Story:
                return azureDevOpsWorkItemType_1.AzureDevOpsWorkItemType.UserStory;
            case models_1.BacklogItemType.Feature:
                return azureDevOpsWorkItemType_1.AzureDevOpsWorkItemType.Feature;
            case models_1.BacklogItemType.Epic:
                return azureDevOpsWorkItemType_1.AzureDevOpsWorkItemType.Epic;
            case models_1.BacklogItemType.Bug:
                return azureDevOpsWorkItemType_1.AzureDevOpsWorkItemType.Bug;
            default:
                throw new Error(`Unsupported backlog item type: ${backlogItemType}`);
        }
    }
    static getBacklogItemType(workItemType) {
        switch (workItemType) {
            case azureDevOpsWorkItemType_1.AzureDevOpsWorkItemType.Task:
                return models_1.BacklogItemType.Task;
            case azureDevOpsWorkItemType_1.AzureDevOpsWorkItemType.UserStory:
                return models_1.BacklogItemType.Story;
            case azureDevOpsWorkItemType_1.AzureDevOpsWorkItemType.Feature:
                return models_1.BacklogItemType.Feature;
            case azureDevOpsWorkItemType_1.AzureDevOpsWorkItemType.Epic:
                return models_1.BacklogItemType.Epic;
            case azureDevOpsWorkItemType_1.AzureDevOpsWorkItemType.Bug:
                return models_1.BacklogItemType.Bug;
            default:
                throw new Error(`Unsupported backlog item type: ${workItemType}`);
        }
    }
    static getProviderOptionsWithConfigDefaults(providerOptions) {
        const projectName = configUtils_1.Config.getValueWithDefault(constants_1.ConfigKey.AzDOProjectName, providerOptions.projectName);
        if (!projectName) {
            throw new Error("Need to configure an AzDO project name in config file or environment variable");
        }
        const personalAccessToken = configUtils_1.Config.getValueWithDefault(constants_1.ConfigKey.AzDOAccessToken, providerOptions.personalAccessToken);
        if (!personalAccessToken) {
            throw new Error("Need to configure an AzDO personal access token in config file or environment variable");
        }
        const baseUrl = configUtils_1.Config.getValueWithDefault(constants_1.ConfigKey.AzDOBaseUrl, providerOptions.baseUrl);
        if (!baseUrl) {
            throw new Error("Need to configure an AzDO base URL in config file or environment variable");
        }
        return {
            projectName,
            baseUrl,
            personalAccessToken,
        };
    }
    static createFieldValuePairs(backlogItem) {
        const { name, description, acceptanceCriteria } = backlogItem;
        return [
            {
                field: azureDevOpsFieldName_1.AzureDevOpsFieldName.title,
                value: name,
            },
            {
                field: azureDevOpsFieldName_1.AzureDevOpsFieldName.description,
                value: description,
            },
            {
                field: azureDevOpsFieldName_1.AzureDevOpsFieldName.acceptanceCriteria,
                value: this.stringifyAcceptanceCriteria(acceptanceCriteria),
            },
        ];
    }
    static stringifyAcceptanceCriteria(acceptanceCriteria) {
        if (!acceptanceCriteria) {
            return undefined;
        }
        const acceptanceCriteriaLines = acceptanceCriteria.map((line) => `<li>${line}</li>`).join("");
        return `<div><ul>${acceptanceCriteriaLines}</ul></div>`;
    }
}
exports.AzureDevOpsUtils = AzureDevOpsUtils;
